/*****************************************************************************
 *  HAMEG DMM Instrument Driver
 *  LabWindows/CVI Instrument Driver
 *  Original Release: 2013-07-04
 *  By: Jiri Kominek, Petr Marcalik
 *
 *  Revision: 1.5.0
 *
 *  Modification History:
 *
 *       See ChangeLog
 *
 *****************************************************************************/
#include "hmdmm.h"

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString functionArr[] = {"VOLT:DC", "VOLT:AC", "CURR:DC", "CURR:AC", "RES", "FRES", "FREQ", "TEMP", "CAP", "CONT", "DIOD", "FREQ:CURR", NULL};
static ViString transducerTypeArr[] = {"RTD", "FRTD",NULL};
static ViString sensorTypeArr [] = {"PT100", "PT500", "PT1000",NULL};
static ViString storageLocationArr [] = {"INT", "EXT",NULL};

/*****************************************************************************
*- User-Callable Functions (Exportable Functions) --------------------------*
*****************************************************************************/

/// HIFN VXIplug&play required function. Calls the hmdmm_InitWithOptions function.
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize
/// HIPAR IDQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the initialization procedure.
/// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the instrument in all subsequent instrument driver function calls.
ViStatus _VI_FUNC hmdmm_init(ViRsrc resourceName, ViBoolean IDQuery,
                             ViBoolean resetDevice, ViSession* newInstrSession)
{
	ViStatus error = VI_SUCCESS;

	if (newInstrSession == NULL)
	{
		(void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
		                          VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

		checkErr(RS_ERROR_INVALID_PARAMETER);
	}

	checkErr(hmdmm_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
	return error;
}

/// HIFN This function creates a new RS session and calls the RsInit function.
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize
/// HIPAR IDQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the initialization procedure.
/// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the instrument in all subsequent instrument driver function calls.
ViStatus _VI_FUNC hmdmm_InitWithOptions(
	ViRsrc resourceName,
	ViBoolean IDQuery,
	ViBoolean resetDevice,
	ViString optionString,
	ViSession* newInstrSession
)
{
	ViStatus error = VI_SUCCESS;
	RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
	RsCoreSessionPtr sessionProperties = NULL;
	ViChar completeOptionString[RS_MAX_MESSAGE_BUF_SIZE];

	if (newInstrSession == NULL)
	{
		(void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5, "Null address for Instrument Handle");
		checkErr(RS_ERROR_INVALID_PARAMETER);
	}

	attrList = (RsCoreAttributePtr *)hmdmm_RsCoreAttributeList;

	// USB session to non-vxi capable
	(void)RsCore_CopyToUserBufferAsciiData(completeOptionString, RS_MAX_MESSAGE_BUF_SIZE, optionString);
	(void)RsCore_AppendToCsvString(completeOptionString, ",", RS_MAX_MESSAGE_BUF_SIZE, "AdditionalFlags=1");

    *newInstrSession = 0;

	checkErr(RsCore_NewSpecificDriver(
		resourceName, // VISA Resource Name
		"hmdmm", // driver prefix
		completeOptionString, // Init Options string, is applied to the session settings
		attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        hmdmm_RsCoreAttributeListLength, // Length of the attributes list
		VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "SMW200A", VI_FALSE: "SMW" Used in CheckInstrument()
		0, // WriteDelay
		0, // ReadDelay
		10000, // IO Segment Size
		RS_VAL_OPCWAIT_STBPOLLINGSLOW, // OPC Wait Mode
		HMDMM_OPC_TIMEOUT, // OPC timeout
		10000, // VISA Timeout
		600000, // Self-test timeout
		RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
		RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
		&instrSession));

	// Minimal allowed rscore version is 4.7.0
	checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 7, 0));

	checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

	if (sessionProperties->sessionType == RS_INTF_USB)
	{
		sessionProperties->allowViClear = VI_FALSE;
		sessionProperties->assureWriteWithLF = VI_TRUE;
		//sessionProperties->vxiCapable = VI_FALSE;
	}

	if (sessionProperties->sessionType == RS_INTF_ASRL)
	{
		sessionProperties->allowViClear = VI_FALSE;
	}

	// No SCPI command has been sent yet.
	// Call viClear before sending any SCPI command
	checkErr(RsCore_ViClear(instrSession));

	/* --- Here perform settings that are default for this driver,
	but can be overwritten by the optionsString settings */

	sessionProperties->addTermCharToWriteBinBlock = VI_TRUE;
	sessionProperties->autoSystErrQuery = VI_FALSE;

	// Parse option string and optionally sets the initial state of the following session attributes
	checkErr(RsCore_ApplyOptionString(instrSession, optionString));
	checkErr(RsCore_BuildRepCapTable(instrSession, hmdmm_RsCoreRepCapTable, hmdmm_RsCoreRepCapTableCount));

	// Query *IDN? string, parse it and set the following attributes:
	// -RS_ATTR_INSTRUMENT_MANUFACTURER
	// - RS_ATTR_INSTRUMENT_MODEL - based on the sessionProperties->idnModelFullName true: "HMC8012", false : "HMC"
	// - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
	checkErr(RsCore_QueryAndParseIDNstring(instrSession, HMDMM_SIMULATION_ID_QUERY, NULL));

	// Default Instrument Setup + optional *RST
	if (resetDevice == VI_TRUE)
	{
		checkErr(hmdmm_reset(instrSession));
	}
	else
	{
		checkErr(hmdmm_DefaultInstrSetup(instrSession));
	}

	if (IDQuery == VI_TRUE)
	{
		checkErr(RsCore_FitsIDNpattern(instrSession, HMDMM_VALID_ID_RESPONSE_STRING, HMDMM_ALTERNATIVE_VALID_ID_RESPONSE_STRING));
	}

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_AfterErrorInInit(instrSession, sessionProperties, error, newInstrSession);
	return error;
}

/// HIFN  Configures the Function, AutoRange and Resolution properties.  If the
/// HIFN  value of the Auto Range parameter is  'On', then the Resolution
/// HIFN  parameter is ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Selects the measurement function (all function-related measurement
/// HIPAR measFunction/controls are retained).
/// HIPAR autorange/Disables or enables autoranging for measurements.
/// HIPAR range/Configures a fixed measurement range. This control is ignored if
/// HIPAR range/autorange is enabled. The value entered in this control is used to
/// HIPAR range/select between the ranges listed below.
/// HIPAR resolution/Selects the measurement resolution for measurements. Specify the
/// HIPAR resolution/resolution in the same units as the selected measurement function, not
/// HIPAR resolution/in number of digits
ViStatus _VI_FUNC hmdmm_Configure(ViSession instrSession,
                                  ViInt32   measFunction,
                                  ViBoolean autorange,
                                  ViReal64  range,
                                  ViReal64  resolution)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, measFunction, HMDMM_VAL_DC_VOLTS, HMDMM_VAL_FREQ_CURRENT),
			2, "Function");

	if (measFunction == HMDMM_VAL_CONTINUITY || measFunction == HMDMM_VAL_DIODE)
	{
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:%s", functionArr[measFunction]);
	}
	else
	{
		if (autorange == VI_TRUE)
		{
			//if (measFunction != HMDMM_VAL_FREQUENCY)
			//  sprintf (buffer, "CONF:%s AUTO,%le", functionArr[measFunction], resolution);
			//else
			snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:%s AUTO,%le", functionArr[measFunction], resolution);
		}
		else
			snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:%s %le,%le", functionArr[measFunction], range, resolution);
	}

	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the parameters of the Trigger Subsystem.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerMode/Select the mode of the trigger.
/// HIPAR autoMode/Select the auto mode of the trigger.
/// HIPAR threshold/Sets the level on which a trigger occurs when the trigger source is
/// HIPAR threshold/set to INTernal.
/// HIPAR interval/Sets trigger interval.
/// HIPAR count/Selects the number of triggers that will be accepted by the meter
/// HIPAR count/before returning to the "idle" trigger state.
ViStatus _VI_FUNC hmdmm_Trigger(ViSession instrSession,
                                ViInt32   triggerMode,
                                ViInt32   autoMode,
                                ViReal64  threshold,
                                ViReal64  interval,
                                ViInt32   count)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_TRIGGER_MODE, triggerMode),
			2, "Trigger Mode");

	if (triggerMode == HMDMM_VAL_TRIGGER_AUTO)
	{
		viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_TRIGGER_AUTO_MODE, autoMode),
				3, "Auto Mode");

		if (autoMode != HMDMM_VAL_TRIGGER_AUTO_CONTINUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_TRIGGER_THRESHOLD, threshold),
				4, "Threshold");

		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_TRIGGER_INTERVAL, interval),
				5, "Interval");
	}

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_TRIGGER_COUNT, count),
			6, "Count");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the temperature function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR ptType/Selects the sensor type.
/// HIPAR rtdConnection/Selects the transducer type.
/// HIPAR resolution/Selects the measurement resolution for measurements. Specify the
/// HIPAR resolution/resolution in the same units as the selected measurement function, not
/// HIPAR resolution/in number of digits
/// HIPAR temperatureUnit/Selects the units to be used for temperature measurements.
ViStatus _VI_FUNC hmdmm_Temperature(ViSession instrSession,
                                    ViInt32   ptType,
                                    ViInt32   rtdConnection,
                                    ViReal64  resolution,
                                    ViInt32   temperatureUnit)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
	         resolution;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, ptType, HMDMM_VAL_SENSOR_PT100, HMDMM_VAL_SENSOR_PT1000),
			2, "PT Type");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, rtdConnection, HMDMM_VAL_PROBE_RTD, HMDMM_VAL_PROBE_FRTD),
			3, "RTD Connection");

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_TEMPERATURE_UNIT, temperatureUnit),
			5, "Temperature Unit");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:TEMP %s,%s", transducerTypeArr[rtdConnection], sensorTypeArr[ptType]);

	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the aperture time for frequency and period measurements.
/// HIFN  Aperture time for frequency and period is analogous to integration time
/// HIFN  for DC and resistance measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Specifies the measurement function for which the aperture time will be
/// HIPAR measFunction/configured.
/// HIPAR apertureTime/Selects the aperture time (also called gate time) for frequency or
/// HIPAR apertureTime/period measurements.
ViStatus _VI_FUNC hmdmm_ApertureTime(ViSession instrSession,
                                     ViInt32   measFunction,
                                     ViReal64  apertureTime)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	switch (measFunction)
	{
	case HMDMM_VAL_FREQUENCY:
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_FREQ_APERTURE_TIME, apertureTime),
				3, "Aperture Time");

		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, measFunction), 2, "Function");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Configure input resistance, detector bandwidth and autozero.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Selects the measurement function (all function-related measurement
/// HIPAR measFunction/controls are retained).
/// HIPAR detectorBandwidth/Selects the bandwidth of AC measurement.
ViStatus _VI_FUNC hmdmm_ACFilter(ViSession instrSession,
                                 ViInt32   measFunction,
                                 ViInt32   detectorBandwidth)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	switch (measFunction)
	{
	case HMDMM_VAL_AC_VOLTS:
		viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_AC_VOLTAGE_DETECTOR_BANDWIDTH, detectorBandwidth),
				3, "Detector Bandwidth");

		break;
	case HMDMM_VAL_AC_CURRENT:
		viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_AC_CURRENT_DETECTOR_BANDWIDTH, detectorBandwidth),
				3, "Detector Bandwidth");

		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, measFunction), 2, "Function");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Disables or enables the autozero mode for DC voltage measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autozero/Disables or enables the autozero mode for DC voltage measurements.
ViStatus _VI_FUNC hmdmm_Autozero(ViSession instrSession,
                                 ViInt32   autozero)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_DC_VOLTAGE_AUTOZERO, autozero),
			2, "Autozero");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Turns the null function on or off for selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Selects the measurement function (all function-related measurement
/// HIPAR measFunction/controls are retained).
/// HIPAR nullFunction/Turns the null function on or off.
/// HIPAR nullFunctionValue/Sets the null value for selected measurements.
ViStatus _VI_FUNC hmdmm_NullFunction(ViSession instrSession,
                                     ViInt32   measFunction,
                                     ViBoolean nullFunction,
                                     ViReal64  nullFunctionValue)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	switch (measFunction)
	{
	case HMDMM_VAL_DC_VOLTS:
		if (nullFunction == VI_TRUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_DC_VOLTAGE_NULL, nullFunctionValue),
				4, "Null Function Value");

		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_DC_VOLTAGE_NULL_FUNCTION_STATE, nullFunction),
				3, "Null Function");
		break;

	case HMDMM_VAL_AC_VOLTS:
		if (nullFunction == VI_TRUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_AC_VOLTAGE_NULL, nullFunctionValue),
				4, "Null Function Value");

		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_AC_VOLTAGE_NULL_FUNCTION_STATE, nullFunction),
				3, "Null Function");
		break;

	case HMDMM_VAL_AC_CURRENT:
		if (nullFunction == VI_TRUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_AC_CURRENT_NULL, nullFunctionValue),
				4, "Null Function Value");

		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_AC_CURRENT_NULL_FUNCTION_STATE, nullFunction),
				3, "Null Function");
		break;

	case HMDMM_VAL_DC_CURRENT:
		if (nullFunction == VI_TRUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_DC_CURRENT_NULL, nullFunctionValue),
				4, "Null Function Value");

		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_DC_CURRENT_NULL_FUNCTION_STATE, nullFunction),
				3, "Null Function");
		break;

	case HMDMM_VAL_2_WIRE_RES:
		if (nullFunction == VI_TRUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_TWO_WIRES_RESISTANCE_NULL, nullFunctionValue),
				4, "Null Function Value");

		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_TWO_WIRES_RESISTANCE_NULL_FUNCTION_STATE, nullFunction),
				3, "Null Function");
		break;

	case HMDMM_VAL_4_WIRE_RES:
		if (nullFunction == VI_TRUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_FOUR_WIRES_RESISTANCE_NULL, nullFunctionValue),
				4, "Null Function Value");

		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_FOUR_WIRES_RESISTANCE_NULL_FUNCTION_STATE, nullFunction),
				3, "Null Function");
		break;

	case HMDMM_VAL_TEMPERATURE:
		if (nullFunction == VI_TRUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_TEMPERATURE_NULL, nullFunctionValue),
				4, "Null Function Value");

		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_TEMPERATURE_NULL_FUNCTION_STATE, nullFunction),
				3, "Null Function");
		break;

	case HMDMM_VAL_CAPACITANCE:
		if (nullFunction == VI_TRUE)
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_CAPACITANCE_NULL, nullFunctionValue),
				4, "Null Function Value");

		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_CAPACITANCE_NULL_FUNCTION_STATE, nullFunction),
				3, "Null Function");
		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, measFunction), 2, "Function");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the beeper of the measurement function.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     HMDMM_ATTR_CONTINUITY_BEEPER
/// HIFN     HMDMM_ATTR_DIODE_BEEPER
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]CONTinuity:BEEPer[:STATe]
/// HIFN     [SENSe:]DIODe:BEEPer[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Selects the measurement function.
/// HIPAR beeper/Activates or disables the beeper of the measurement function.
ViStatus _VI_FUNC hmdmm_BeeperMeasurementState(ViSession instrSession,
                                               ViInt32   measFunction,
                                               ViBoolean beeper)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	switch (measFunction)
	{
	case HMDMM_VAL_CONTINUITY:
		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_CONTINUITY_BEEPER, beeper),
				3, "Beeper");

		break;

	case HMDMM_VAL_DIODE:
		viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_DIODE_BEEPER, beeper),
				3, "Beeper");

		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, measFunction), 2, "Function");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the threshold of the measurement function.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     HMDMM_ATTR_CONTINUITY_THRESHOLD
/// HIFN     HMDMM_ATTR_DIODE_THRESHOLD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]CONTinuity:THReshold
/// HIFN     [SENSe:]DIODe:THReshold
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Selects the measurement function.
/// HIPAR threshold/Sets the threshold of the diode/continuity measurement function. In
/// HIPAR threshold/this case, the instrument does not switch into continuity mode. For
/// HIPAR threshold/activating the continuity measurement function use the FUNC command.
ViStatus _VI_FUNC hmdmm_ThresholdMeasurement(ViSession instrSession,
                                             ViInt32   measFunction,
                                             ViReal64  threshold)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	switch (measFunction)
	{
	case HMDMM_VAL_CONTINUITY:
		viCheckParm(RsCore_InvalidViReal64Range(instrSession, threshold, 0.0, 1.0e6), 3, "Threshold");

		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_CONTINUITY_THRESHOLD, threshold),
				3, "Threshold");

		break;

	case HMDMM_VAL_DIODE:
		viCheckParm(RsCore_InvalidViReal64Range(instrSession, threshold, 0.0, 5.0), 3, "Threshold");

		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_DIODE_THRESHOLD, threshold),
				3, "Threshold");

		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, measFunction), 2, "Function");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns the Width and Height of the screenshot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR width/Returns the width of the screenshot.
/// HIPAR height/Returns the height of the screenshot.
ViStatus _VI_FUNC hmdmm_HardcopyScreenshotSize(ViSession instrSession,
                                               ViInt32*  width,
                                               ViInt32*  height)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_HARDCOPY_SIZE_X, width),
			2, "Width");

	viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_HARDCOPY_SIZE_Y, height),
			3, "Height");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function selects the data format of the screenshot and queries
/// HIFN  responses screenshot data in binary format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR hardcopyFormat/This parameter selects the data format of the screenshot.
/// HIPAR fileName/This control selects file for the data transfer from instrument to
/// HIPAR fileName/control computer.
ViStatus _VI_FUNC hmdmm_HardcopyData(ViSession instrSession,
                                     ViInt32   hardcopyFormat,
                                     ViString  fileName)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_HARDCOPY_FORMAT, hardcopyFormat),
			3, "Hardcopy Format");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, fileName), 3, "File Name");

	checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, "HCOPY:DATA?", fileName, RS_VAL_TRUNCATE));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);

	return error;
}


/// HIFN  This function turns the data logging function on or off.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     HMDMM_ATTR_DATA_LOGGING
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DATA:LOG[:STATe]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Turns the data logging function on or off.
ViStatus _VI_FUNC hmdmm_DataLogging(ViSession instrSession,
                                    ViBoolean state)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_DATA_LOGGING, state),
			2, "State");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function defines the parameters for the logging function.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     HMDMM_ATTR_DATA_LOGGING_FILE_FORMAT
/// HIFN     HMDMM_ATTR_DATA_LOGGING_MODE
/// HIFN     HMDMM_ATTR_DATA_LOGGING_TIME
/// HIFN     HMDMM_ATTR_DATA_LOGGING_MEASUREMENT_COUNT
/// HIFN     HMDMM_ATTR_DATA_LOGGING_MEASUREMENT_INTERVAL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DATA:LOG:FORMat
/// HIFN     DATA:LOG:MODE
/// HIFN     DATA:LOG:TIME
/// HIFN     DATA:LOG:COUNt
/// HIFN     DATA:LOG:INTerval
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileFormat/Defines the data logging file format.
/// HIPAR loggingMode/Selects the data logging mode.
/// HIPAR dataLoggingTime/Sets the data logging time.
/// HIPAR numberofMeasurement/Sets the number of measurement values to be captured.
/// HIPAR measurementInterval/Selects a logging measurement interval. The measurement interval
/// HIPAR measurementInterval/describes the time betweenthe recorded measurements.
ViStatus _VI_FUNC hmdmm_ConfigureDataLogging(ViSession instrSession,
                                             ViInt32   fileFormat,
                                             ViInt32   loggingMode,
                                             ViReal64  dataLoggingTime,
                                             ViInt32   numberofMeasurement,
                                             ViReal64  measurementInterval)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_DATA_LOGGING_FILE_FORMAT, fileFormat),
			2, "File Format");

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_DATA_LOGGING_MODE, loggingMode),
			2, "Logging Mode");

	switch (loggingMode)
	{
	case HMDMM_VAL_DATA_LOG_MODE_UNLIMITED:
		break;

	case HMDMM_VAL_DATA_LOG_MODE_COUNT:
		viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_DATA_LOGGING_MEASUREMENT_COUNT, numberofMeasurement),
				5, "Number of Measurement");
		break;

	case HMDMM_VAL_DATA_LOG_MODE_TIME:
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_DATA_LOGGING_TIME, dataLoggingTime),
				4, "Data Logging Time");
		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, loggingMode), 3, "Logging Mode");
	}

	viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_DATA_LOGGING_MEASUREMENT_INTERVAL, measurementInterval),
			2, "Measurement Interval");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function defines the file name and storage location for the
/// HIFN  logging function.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DATA:LOG:FNAMe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Defines the file name for the logging function.
/// HIPAR storageLocation/Defines the data storage location for the logging function.
ViStatus _VI_FUNC hmdmm_SetDataLoggingFileName(ViSession instrSession,
                                               ViString  fileName,
                                               ViInt32   storageLocation)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, storageLocation, HMDMM_VAL_STORAGE_LOCATION_INT, HMDMM_VAL_STORAGE_LOCATION_EXT),
			3, "Storage Location");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:LOG:FNAM \"%s\",%s", fileName, storageLocationArr[storageLocation]);

	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function returns the logging file data values of the selected
/// HIFN  storage location and file name.
/// HIFN
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DATA:DATA?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Defines the file name for the logging function.
/// HIPAR storageLocation/Defines the data storage location for the logging function.
/// HIPAR bufferSize/This control defines size of array passed to Logging File Data.
/// HIPAR loggingFileData/Returns the logging file data values of the selected storage location
/// HIPAR loggingFileData/and file name.
ViStatus _VI_FUNC hmdmm_ReadLoggingFileData(ViSession instrSession,
                                            ViString  fileName,
                                            ViInt32   storageLocation,
                                            ViInt32   bufferSize,
                                            ViChar    loggingFileData[],
                                            ViInt32   timeout)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
	ViUInt32 oldTimeout = 0;

	checkErr(RsCore_LockSession(instrSession));

	if (timeout > 0)
	{
		checkErr(hmdmm_GetVISATimeout(instrSession, &oldTimeout));
		checkErr(hmdmm_SetVISATimeout(instrSession, timeout));
	}

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, storageLocation, HMDMM_VAL_STORAGE_LOCATION_INT, HMDMM_VAL_STORAGE_LOCATION_EXT),
			3, "Storage Location");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, loggingFileData), 5, "Logging File Data");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:DATA? \"%s\",%s", fileName, storageLocationArr[storageLocation]);

	checkErr(hmdmm_QueryViString(instrSession, cmd, bufferSize, loggingFileData));

	if (oldTimeout > 0)
		hmdmm_SetVISATimeout(instrSession, oldTimeout);

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function deletes the logging file data values of the selected
/// HIFN  storage location and file name. If no storage location is selected, the
/// HIFN  instrument uses the internal memory. Please notice that the logging
/// HIFN  function has to be activated, if you want to use the manual or single
/// HIFN  trigger mode (trigger via TRIG button). Without activating the logging
/// HIFN  function in single or manual trigger mode, the instrument is not able
/// HIFN  to save a logging file internally or on the USB stick.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DATA:DELete
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Defines the file name for the logging function.
/// HIPAR storageLocation/Defines the data storage location for the logging function.
ViStatus _VI_FUNC hmdmm_DeleteLoggingFileData(ViSession instrSession,
                                              ViString  fileName,
                                              ViInt32   storageLocation)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, storageLocation, HMDMM_VAL_STORAGE_LOCATION_INT, HMDMM_VAL_STORAGE_LOCATION_EXT),
			3, "Storage Location");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:DEL \"%s\",%s", fileName, storageLocationArr[storageLocation]);

	checkErr(RsCore_Write(instrSession, cmd));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function returns the number of log file values of the selected
/// HIFN  storage location and file name.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DATA:POINts?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Defines the file name for the logging function.
/// HIPAR storageLocation/Defines the data storage location for the logging function.
/// HIPAR numberOfLoggingValues/Returns the number of log file values of the selected storage location
/// HIPAR numberOfLoggingValues/and file name.
ViStatus _VI_FUNC hmdmm_GetNumberOfLoggingValues(ViSession instrSession,
                                                 ViString  fileName,
                                                 ViInt32   storageLocation,
                                                 ViInt32*  numberOfLoggingValues)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, storageLocation, HMDMM_VAL_STORAGE_LOCATION_INT, HMDMM_VAL_STORAGE_LOCATION_EXT),
			3, "Storage Location");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:POIN? \"%s\",%s", fileName, storageLocationArr[storageLocation]);

	viCheckParm(RsCore_InvalidNullPointer(instrSession, numberOfLoggingValues), 4, "Number Of Logging Values");

	checkErr(hmdmm_QueryViInt32(instrSession, cmd, numberOfLoggingValues));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function returns all saved logging files of the selected storage
/// HIFN  location. If no storage location is selected, the instrument queries
/// HIFN  the internal memory. Please notice that the logging function has to be
/// HIFN  activated, if you want to use the manual or single trigger mode
/// HIFN  (trigger via TRIG button). Without activating the logging function in
/// HIFN  single or manual trigger mode, the instrument is not able to save a
/// HIFN  logging file internally or on the USB stick. If you store the logging
/// HIFN  file on the USB stick, the query returns all files depending on the
/// HIFN  storage format (CSV or TXT files).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DATA:LIST?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR storageLocation/Defines the data storage location for the logging function.
/// HIPAR bufferSize/This control defines size of array passed to Logging File.
/// HIPAR loggingFiles/Returns all saved logging files of the selected storage location.
ViStatus _VI_FUNC hmdmm_ListAllLoggingFilesData(ViSession instrSession,
                                                ViInt32   storageLocation,
                                                ViInt32   bufferSize,
                                                ViChar    loggingFiles[])
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, storageLocation, HMDMM_VAL_STORAGE_LOCATION_INT, HMDMM_VAL_STORAGE_LOCATION_EXT),
			3, "Storage Location");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:LIST? %s", storageLocationArr[storageLocation]);

	viCheckParm(RsCore_InvalidNullPointer(instrSession, loggingFiles), 5, "Logging Files");

	checkErr(hmdmm_QueryViString(instrSession, cmd, bufferSize, loggingFiles));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Selects the measurement function (all function-related measurement
/// HIFN  controls are retained).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Selects the measurement function (all function-related measurement
/// HIPAR measFunction/controls are retained).
ViStatus _VI_FUNC hmdmm_MeasurementFunction(ViSession instrSession,
                                            ViInt32   measFunction)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_FUNCTION, measFunction),
			2, "Function");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Configures the Function, AutoRange and Resolution properties.  If the
/// HIFN  value of the Auto Range parameter is  'On', then the Resolution
/// HIFN  parameter is ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Selects the measurement function (all function-related measurement
/// HIPAR measFunction/controls are retained).
/// HIPAR autorange/Disables or enables autoranging for measurements.
/// HIPAR range/Configures a fixed measurement range. This control is ignored if
/// HIPAR range/autorange is enabled. The value entered in this control is used to
/// HIPAR range/select between the ranges listed below.
/// HIPAR resolution/Selects the measurement resolution for measurements. Specify the
/// HIPAR resolution/resolution in the same units as the selected measurement function, not
/// HIPAR resolution/in number of digits
ViStatus _VI_FUNC hmdmm_SetRange(ViSession instrSession,
                                 ViInt32   measFunction,
                                 ViBoolean autorange,
                                 ViReal64  range,
                                 ViReal64  resolution)
{
	ViStatus error = VI_SUCCESS;
	         resolution;

	checkErr(RsCore_LockSession(instrSession));

	switch (measFunction)
	{
	case HMDMM_VAL_DC_VOLTS:

		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_DC_VOLTAGE_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_DC_VOLTAGE_AUTO_RANGE, autorange),
					3, "Autorange");
		}

		break;

	case HMDMM_VAL_AC_VOLTS:
		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_AC_VOLTAGE_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_AC_VOLTAGE_AUTO_RANGE, autorange),
					3, "Autorange");
		}

		break;

	case HMDMM_VAL_DC_CURRENT:

		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_DC_CURRENT_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_DC_CURRENT_AUTO_RANGE, autorange),
					3, "Autorange");
		}

		break;

	case HMDMM_VAL_AC_CURRENT:

		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_AC_CURRENT_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_AC_CURRENT_AUTO_RANGE, autorange),
					3, "Autorange");
		}
		break;

	case HMDMM_VAL_2_WIRE_RES:

		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_TWO_WIRES_RESISTANCE_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_TWO_WIRES_RESISTANCE_AUTO_RANGE, autorange),
					3, "Autorange");
		}
		break;

	case HMDMM_VAL_4_WIRE_RES:

		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_FOUR_WIRES_RESISTANCE_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_FOUR_WIRES_RESISTANCE_AUTO_RANGE, autorange),
					3, "Autorange");
		}
		break;

	case HMDMM_VAL_FREQUENCY:

		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_FREQ_VOLTAGE_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_AUTO_FREQ_VOLTAGE_RANGE, autorange),
					3, "Autorange");
		}
		break;

	case HMDMM_VAL_CAPACITANCE:

		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_CAPACITANCE_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_CAPACITANCE_AUTO_RANGE, autorange),
					3, "Autorange");
		}
		break;

	case HMDMM_VAL_FREQ_CURRENT:

		if (autorange == VI_FALSE)
		{
			viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_FREQ_CURRENT_RANGE, range),
					4, "Range");
		}
		else
		{
			viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_FREQ_CURRENT_AUTO_RANGE, autorange),
					3, "Autorange");
		}
		break;
	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, measFunction), 2, "Function");
		break;
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Configures the RTD/transducer probe type for temperature measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sensorType/Selects the sensor type.
/// HIPAR transducerType/Selects the transducer type.
ViStatus _VI_FUNC hmdmm_TemperatureTransducerType(ViSession instrSession,
                                                  ViInt32   sensorType,
                                                  ViInt32   transducerType)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_TEMPERATURE_RTD_TYPE, sensorType),
			2, "Sensor Type");

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_TEMPERATURE_PROBE_TYPE, transducerType),
			3, "Transducer Type");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Changes the state of the triggering system from the "idle" state to
/// HIFN  the "wait-for-trigger" state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC hmdmm_WaitForTrigger(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_SetAttributeViString(instrSession, "", HMDMM_ATTR_INITIATE, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  When the instrument trigger source is configured for software trigger,
/// HIFN  this panel triggers the instrument once.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC hmdmm_SoftwareTrigger(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_SetAttributeViString(instrSession, "", HMDMM_ATTR_SEND_SOFTWARE_TRIGGER, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Clears all values from the statistics register.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC hmdmm_AverageClear(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_SetAttributeViString(instrSession, "", HMDMM_ATTR_AVERAGE_CLEAR, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the beeper on the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR beeperState/Disables or enables the beeper tone.
/// HIPAR beep/Issues a single beep immediately from the instrument.
ViStatus _VI_FUNC hmdmm_Beeper(ViSession instrSession,
                               ViBoolean beeperState,
                               ViBoolean beep)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_BEEPER_STATE, beeperState),
			2, "Beeper State");

	if (beep == VI_TRUE)
	viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_BEEP, beep),
			3, "Beep");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Configures the ADC rate for the activated measurement function.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     HMDMM_ATTR_ADC_RATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]ADCRate
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR adcRate/Selects the ADC rate for the activated measurement function. The
/// HIPAR adcRate/reading rate per second of the ADC rate function is depending on the
/// HIPAR adcRate/activated measurement function.
ViStatus _VI_FUNC hmdmm_ADCRate(ViSession instrSession,
                                ViInt32   adcRate)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_ADC_RATE, adcRate),
			2, "ADC Rate");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Configures the Function and reads measurement value, AutoRange and Resolution properties.  If the
/// HIFN  value of the Auto Range parameter is  'On', then the Resolution
/// HIFN  parameter is ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measFunction/Selects the measurement function (all function-related measurement
/// HIPAR measFunction/controls are retained).
/// HIPAR autorange/Disables or enables autoranging for measurements.
/// HIPAR range/Configures a fixed measurement range. This control is ignored if
/// HIPAR range/autorange is enabled. The value entered in this control is used to
/// HIPAR range/select between the ranges listed below.
/// HIPAR resolution/Selects the measurement resolution for measurements. Specify the
/// HIPAR resolution/resolution in the same units as the selected measurement function, not
/// HIPAR resolution/in number of digits
/// HIPAR measValue/Returns the measurement value.
ViStatus _VI_FUNC hmdmm_Measure(ViSession instrSession,
                                ViInt32   measFunction,
                                ViBoolean autorange,
                                ViReal64  range,
                                ViReal64  resolution,
                                ViReal64* measValue)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, measFunction, HMDMM_VAL_DC_VOLTS, HMDMM_VAL_FREQ_CURRENT),
			2, "Function");

	if (measFunction == HMDMM_VAL_TEMPERATURE)
	{
		snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS:%s?", functionArr[measFunction]);
	}
	else
	{
		if (measFunction == HMDMM_VAL_CONTINUITY || measFunction == HMDMM_VAL_DIODE)
		{
			snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS:%s?", functionArr[measFunction]);
		}
		else
		{
			if (autorange == VI_TRUE)
			{
				if (measFunction == HMDMM_VAL_FREQUENCY)
					snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS:%s? AUTO,%le", functionArr[measFunction], resolution);
				else
					snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS:%s? AUTO,%le", functionArr[measFunction], resolution);
			}
			else
			{
				snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS:%s? %le,%le", functionArr[measFunction], range, resolution);
			}
		}
	}

	checkErr(hmdmm_MeasureRepeated(instrSession, cmd, 0, measValue));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Returns a measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurement/Triggers and returns a measurement.
ViStatus _VI_FUNC hmdmm_FetchMeasurement(ViSession instrSession,
                                         ViInt32   timeout,
                                         ViReal64* measurement)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_MeasureRepeated(instrSession, "FETC?", timeout, measurement));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Returns a measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurement/Triggers and returns a measurement.
ViStatus _VI_FUNC hmdmm_FetchSecondFunctionMeasurement(ViSession instrSession,
                                         ViInt32   timeout,
                                         ViReal64* measurement)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_MeasureRepeated(instrSession, "FETC2?", timeout, measurement));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Puts the instrument into the wait-for-trigger state and waits until
/// HIFN  the trigger has been received, then returns a single reading. This
/// HIFN  function uses synchronous techniques to determine when the measurement
/// HIFN  is complete. Using this method the function will wait while the
/// HIFN  measurement is in progress. The advantage of this method is that the
/// HIFN  user can use one function to enter the wait-for-trigger state and also
/// HIFN  return the reading.  The disadvantage is that no other operation can be
/// HIFN  performed while measurement is in progress.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/Sets the maximum amount of time (in milli seconds) that the controller
/// HIPAR timeout/will wait for the instrument to respond before it times out.
/// HIPAR measurement/Triggers and returns a measurement.
ViStatus _VI_FUNC hmdmm_ReadMeasurement(ViSession instrSession,
                                        ViInt32   timeout,
                                        ViReal64* measurement)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_MeasureRepeated(instrSession, "READ?", timeout, measurement));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Activates the selected math operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mathFunction/Select the math function. Only one function can be enabled at a time.
/// HIPAR state/Disable or enable the selected math function.
ViStatus _VI_FUNC hmdmm_ActiveMathOperation(ViSession instrSession,
                                            ViInt32   mathFunction,
                                            ViBoolean state)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_MATH_FUNCTION, mathFunction),
			2, "Math Function");

	viCheckParm(hmdmm_SetAttributeViBoolean(instrSession, "", HMDMM_ATTR_MATH_ENABLED, state),
			3, "State");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Stores a null value in the multimeter's Null Register.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nullValue/Stores a null value in the multimeter's Null Register. You can set the
/// HIPAR nullValue/value to any number between -120% and +120% of the highest range, for
/// HIPAR nullValue/the present function.
ViStatus _VI_FUNC hmdmm_NullOperation(ViSession instrSession,
                                      ViReal64  nullValue)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_MATH_OFFSET, nullValue),
			2, "Null Value");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Store a relative value in the dB Relative Register.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dbReference/Store a relative value in the dB Relative Register. You must turn on
/// HIPAR dbReference/the math operation before writing to the math register.
ViStatus _VI_FUNC hmdmm_dBMeasurements(ViSession instrSession,
                                       ViReal64  dbReference)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_MATH_DB_REFERENCE, dbReference),
			2, "dB Reference");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function sets the dBm reference value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dbmReference/Select the dBm reference value.
ViStatus _VI_FUNC hmdmm_dBmMeasurements(ViSession instrSession,
                                        ViInt32   dbmReference)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_MATH_DBM_REFERENCE, dbmReference),
			2, "dBm Reference");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Read the average of all readings, number of readings, average minimum
/// HIFN  and maximum value found during a min-max operation. The multimeter
/// HIFN  clears the value when min-max is turned on, when power has been off, or
/// HIFN  after a remote interface reset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR average/Read the average of all readings taken since min-max was enabled. The
/// HIPAR average/multimeter clears the value when min-max is turned on, when power has
/// HIPAR average/been off, or after a remote interface reset.
/// HIPAR averageMin/Read the average of all readings taken since min-max was enabled. The
/// HIPAR averageMin/multimeter clears the value when min-max is turned on, when power has
/// HIPAR averageMin/been off, or after a remote interface reset.
/// HIPAR averageMax/Read the maximum value found during a min-max operation. The
/// HIPAR averageMax/multimeter clears the value when min-max is turned on, when power has
/// HIPAR averageMax/been off, or after a remote interface reset
/// HIPAR averageCount/Read the number of readings taken since min-max was enabled. The
/// HIPAR averageCount/multimeter clears the value when min-max is turned on, when power has
/// HIPAR averageCount/been off, or after a remote interface reset.
ViStatus _VI_FUNC hmdmm_Average(ViSession instrSession,
                                ViReal64* average,
                                ViReal64* averageMin,
                                ViReal64* averageMax,
                                ViReal64* standardDeviation,
                                ViReal64* peakToPeak,
                                ViInt32*  averageCount)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_GetAttributeViReal64(instrSession, "", HMDMM_ATTR_AVERAGE, average),
			2, "Average");

	viCheckParm(hmdmm_GetAttributeViReal64(instrSession, "", HMDMM_ATTR_AVERAGE_MINIMUM, averageMin),
			3, "Average Min");

	viCheckParm(hmdmm_GetAttributeViReal64(instrSession, "", HMDMM_ATTR_AVERAGE_MAXIMUM, averageMax),
			4, "Average Max");

	viCheckParm(hmdmm_GetAttributeViReal64(instrSession, "", HMDMM_ATTR_AVERAGE_STANDARD_DEVIATION, standardDeviation),
			5, "Standard Deviation");

	viCheckParm(hmdmm_GetAttributeViReal64(instrSession, "", HMDMM_ATTR_AVERAGE_PEAK_TO_PEAK, peakToPeak),
			6, "Peak To Peak");

	viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_AVERAGE_COUNT, averageCount),
			7, "Average Count");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  The limit test operation enables you to perform pass/fail testing to
/// HIFN  upper and lower limits that you specify.
/// HIFN     Note:
/// HIFN     (1) This operation applies to all measurement functions, except
/// HIFN  continuity and diode tests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR highLimit/Set the upper limit for limit testing. You can set the value to any
/// HIPAR highLimit/number between -120% and +120% of the highest range, for the present
/// HIPAR highLimit/function.
/// HIPAR lowLimit/Set the lower limit for limit testing. You can set the value to any
/// HIPAR lowLimit/number between -120% and +120% of the highest range, for the present
/// HIPAR lowLimit/function.
ViStatus _VI_FUNC hmdmm_LimitTesting(ViSession instrSession,
                                     ViReal64  highLimit,
                                     ViReal64  lowLimit)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_UPPER_LIMIT, highLimit),
			2, "High Limit");

	viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_LOWER_LIMIT, lowLimit),
			3, "Low Limit");

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Configures the instrument for temperature measurements. The displayed
/// HIFN  values (including statistic
/// HIFN     values) will be reset and the instrument immediately triggers a
/// HIFN  measurement.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     MEASure:TEMPerature?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR ptType/Selects the sensor type.
/// HIPAR rtdConnection/Selects the transducer type.
/// HIPAR measurementValue/Returns the measurement value.
ViStatus _VI_FUNC hmdmm_TemperatureMeasurement(ViSession instrSession,
                                               ViInt32   ptType,
                                               ViInt32   rtdConnection,
                                               ViReal64* measurementValue)
{
	ViStatus error = VI_SUCCESS;
	ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, ptType, HMDMM_VAL_SENSOR_PT100, HMDMM_VAL_SENSOR_PT1000),
			2, "PT Type");
	viCheckParm(RsCore_InvalidViInt32Range(instrSession, rtdConnection, HMDMM_VAL_PROBE_RTD, HMDMM_VAL_PROBE_FRTD),
			3, "RTD Connection");

	snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MEAS:TEMP? %s,%s", transducerTypeArr[rtdConnection], sensorTypeArr[ptType]);

	checkErr(hmdmm_MeasureRepeated(instrSession, cmd, 0, measurementValue));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/*****************************************************************************
 * Function: hmdmm_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC hmdmm_SetAttributeViInt32(ViSession instrSession,
                                            ViString  channelName,
                                            ViAttr   attributeId,
                                            ViInt32   value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViReal64 */
ViStatus _VI_FUNC hmdmm_SetAttributeViReal64(ViSession instrSession,
                                             ViString  channelName,
                                             ViAttr   attributeId,
                                             ViReal64  value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViString */
ViStatus _VI_FUNC hmdmm_SetAttributeViString(ViSession instrSession,
                                             ViString  channelName,
                                             ViAttr   attributeId,
                                             ViString  value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViBoolean */
ViStatus _VI_FUNC hmdmm_SetAttributeViBoolean(ViSession instrSession,
                                              ViString  channelName,
                                              ViAttr   attributeId,
                                              ViBoolean value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_SetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: hmdmm_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC hmdmm_GetAttributeViInt32(ViSession instrSession,
                                            ViString  channelName,
                                            ViAttr   attributeId,
                                            ViInt32*  value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);
	error = RsCore_GetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViReal64 */
ViStatus _VI_FUNC hmdmm_GetAttributeViReal64(ViSession instrSession,
                                             ViString  channelName,
                                             ViAttr   attributeId,
                                             ViReal64* value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_GetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViString */
ViStatus _VI_FUNC hmdmm_GetAttributeViString(ViSession instrSession,
                                             ViString  channelName,
                                             ViAttr   attributeId,
                                             ViInt32   bufSize,
                                             ViChar    value[])
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_GetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, bufSize, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/* ViBoolean */
ViStatus _VI_FUNC hmdmm_GetAttributeViBoolean(ViSession  instrSession,
                                              ViString   channelName,
                                              ViAttr   attributeId,
                                              ViBoolean* value)
{
	ViStatus error = VI_SUCCESS;

	(void)RsCore_LockSession(instrSession);

	error = RsCore_GetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function returns the repeated capability id(s) that belongs
/// HIFN to the attribute you specify. More then one id is listed in
/// HIFN comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability id(s) belonging to the attribute.
ViStatus _VI_FUNC hmdmm_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViAttr   attributeId,
                                                          ViInt32   bufferSize,
                                                          ViChar    repeatedCapabilityIds[])
{  
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
			3, "Buffer Size");

	checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeId, bufferSize, repeatedCapabilityIds));

Error:
	(void)RsCore_UnlockSession(instrSession);

	return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC hmdmm_GetAttributeRepeatedCapabilityIdNames(
							ViSession instrSession,
							ViAttr   attributeId,
							ViString  repeatedCapabilityId,
							ViInt32   bufferSize,
							ViChar    repeatedCapabilityIdNames[])
{  
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
			4, "Buffer Size");

	checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
		attributeId,
		repeatedCapabilityId,
		bufferSize,
		repeatedCapabilityIdNames));

Error:
	(void)RsCore_UnlockSession(instrSession);

	return error;
}


/// HIFN This function resets the instrument to a known state and sends initialization commands to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
ViStatus _VI_FUNC hmdmm_reset(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	if (RsCore_Simulating(instrSession) == VI_FALSE)
	{
		checkErr(hmdmm_ClearStatus(instrSession));
		checkErr(RsCore_Write(instrSession, "*RST"));
		checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
	}

	checkErr(hmdmm_DefaultInstrSetup(instrSession));
	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);

	return error;
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC hmdmm_self_test(ViSession instrSession,
                                  ViInt16*  testResult,
                                  ViChar    testMessage[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

	checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET /// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC hmdmm_revision_query(ViSession instrSession,
                                       ViChar    driverRev[],
                                       ViChar    instrRev[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, driverRev), 2, "Null address for Instrument Driver Revision");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, instrRev), 3, "Null address for Instrument Revision");

	checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, driverRev));
	checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, instrRev));
	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR errorCode/Returns the error code read from the instrument's error queue.
/// HIPAR errorMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC hmdmm_error_query(ViSession instrSession,
                                    ViInt32*  errorCode,
                                    ViChar    errorMessage[1024])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

	checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN This function converts a status code returned by an instrument driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the status code you specify.
ViStatus _VI_FUNC hmdmm_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[512])
{
	return hmdmm_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN  Configures the front panel display of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayMessage/Display a message on the front panel. The multimeter will display up
/// HIPAR displayMessage/to 12 characters in a message; any additional characters are truncated.
/// HIPAR clearMessage/Clear the message displayed on the front panel.
ViStatus _VI_FUNC hmdmm_Display(ViSession instrSession,
                                ViChar    displayMessage[],
                                ViBoolean clearMessage)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	if (clearMessage == VI_TRUE)
	{
		checkErr(hmdmm_SetAttributeViString(instrSession, "", HMDMM_ATTR_DISPLAY_CLEAR, NULL));
	}
	else
	{
		viCheckParm(hmdmm_SetAttributeViString(instrSession, "", HMDMM_ATTR_DISPLAY_TEXT, displayMessage),
				3, "Display Message");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Configures the front panel display of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR instrumentState/This function sets the instrument to remote or local state.
ViStatus _VI_FUNC hmdmm_RemoteSystemSettings(ViSession instrSession,
                                             ViInt32   instrumentState)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	switch (instrumentState)
	{
	case HMDMM_VAL_TO_REMOTE:
		checkErr(hmdmm_SetAttributeViString(instrSession, "", HMDMM_ATTR_SYSTEM_LOCAL, NULL));

		break;

	case HMDMM_VAL_TO_LOCAL:
		checkErr(hmdmm_SetAttributeViString(instrSession, "", HMDMM_ATTR_SYSTEM_REMOTE, NULL));
		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, instrumentState), 2, "Instrument State");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function stores current instrument setup into indicated save
/// HIFN  position or recalls instrument setup from memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/Selects operation.
/// HIPAR position/Sets th memory bank position.
ViStatus _VI_FUNC hmdmm_StoreAndLoadInstrumentSetup(ViSession instrSession,
                                                    ViInt32   operation,
                                                    ViInt32   position)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	switch (operation)
	{
	case HMDMM_VAL_SAVE:
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_SAVE, position),
				3, "Position");

		break;

	case HMDMM_VAL_RECALL:
		viCheckParm(hmdmm_SetAttributeViReal64(instrSession, "", HMDMM_ATTR_RECALL, position),
				3, "Position");
		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function clears all bits in the Questionable Data enable register
/// HIFN  and the Standard Operation enable register.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.

ViStatus _VI_FUNC hmdmm_PresetRegister(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_SetAttributeViString(instrSession, "", HMDMM_ATTR_PRESET_REGISTER, NULL));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function sets the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerType/This control selects type of the registers that will be configured.
/// HIPAR statusRegister/This control selects the registers that will be configured.
/// HIPAR registerValue/Value of selected register.
ViStatus _VI_FUNC hmdmm_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerType,
                                          ViInt32   statusRegister,
                                          ViInt32   registerValue)
{
	ViStatus error = VI_SUCCESS;
	         statusRegister;

	checkErr(RsCore_LockSession(instrSession));

	switch (registerType)
	{
	case HMDMM_VAL_REG_QUES:
		viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_STATUS_REGISTER_QUESTIONABLE_ENABLE, registerValue),
				4, "Register Value");
		break;

	case HMDMM_VAL_REG_OPER:
		viCheckParm(hmdmm_SetAttributeViInt32(instrSession, "", HMDMM_ATTR_STATUS_REGISTER_OPERATION_ENABLE, registerValue),
				4, "Register Value");
		break;

	default:
		viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerType), 2, "Register Type");
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function queries the status reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerType/This control selects type of the registers that will be queried.
/// HIPAR statusRegister/This control selects the registers that will be queried.
/// HIPAR registerValue/Returns value of selected register.
ViStatus _VI_FUNC hmdmm_GetStatusRegister(ViSession instrSession,
                                          ViInt32   registerType,
                                          ViInt32   statusRegister,
                                          ViInt32*  registerValue)
{
	ViStatus error = VI_SUCCESS;
	         statusRegister;

	checkErr(RsCore_LockSession(instrSession));

	if (registerType == HMDMM_VAL_REG_QUES)
	{
		switch (registerType)
		{
		case HMDMM_VAL_REG_COND:
			viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_STATUS_REGISTER_QUESTIONABLE_CONDITION, registerValue),
					4, "Register Value");
			break;

		case HMDMM_VAL_REG_ENABLE:
			viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_STATUS_REGISTER_QUESTIONABLE_ENABLE, registerValue),
					4, "Register Value");
			break;

		case HMDMM_VAL_REG_EVENT:
			viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_STATUS_REGISTER_QUESTIONABLE_EVENT, registerValue),
					4, "Register Value");
			break;

		default:
			viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerType), 2, "Register Type");
		}
	}
	else
	{
		switch (registerType)
		{
		case HMDMM_VAL_REG_COND:
			viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_STATUS_REGISTER_OPERATION_CONDITION, registerValue),
					4, "Register Value");
			break;

		case HMDMM_VAL_REG_ENABLE:
			viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_STATUS_REGISTER_OPERATION_ENABLE, registerValue),
					4, "Register Value");
			break;

		case HMDMM_VAL_REG_EVENT:
			viCheckParm(hmdmm_GetAttributeViInt32(instrSession, "", HMDMM_ATTR_STATUS_REGISTER_OPERATION_EVENT, registerValue),
					4, "Register Value");
			break;

		default:
			viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerType), 2, "Register Type");
		}
	}

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC hmdmm_ClearStatus(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_ClearStatus(instrSession));
	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC hmdmm_IDQueryResponse(ViSession instrSession,
                                        ViChar    IDQueryResponse[])
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_QueryViString(instrSession, "*IDN?", 256, IDQueryResponse));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC hmdmm_ProcessAllPreviousCommands(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_WriteInstrData(instrSession, "*WAI"));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC hmdmm_QueryOPC(ViSession instrSession,
                                 ViInt32*  OPC)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(hmdmm_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  This function switches ON/OFF various error checkings performed inside
/// HIFN  of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking (reading of
/// HIPAR statusChecking/the status byte and checking it for error). All driver functions are
/// HIPAR statusChecking/using state checking. Switch this function to ON when debugging your
/// HIPAR statusChecking/application. For better bus throughput and instrument performance
/// HIPAR statusChecking/switch it OFF.
ViStatus _VI_FUNC hmdmm_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking)
{
	ViStatus         error = VI_SUCCESS;
	RsCoreSessionPtr rsSession = NULL;

	checkErr(RsCore_GetRsSession(instrSession, &rsSession));
	rsSession->optionChecking = optionChecking;
	checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
	checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
	return error;
}

/// HIFN  This function specifies the minimum timeout value to use (in
/// HIFN  milliseconds) when accessing the device associated with the given
/// HIFN  session.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC hmdmm_SetVISATimeout(ViSession instrSession,
                                       ViUInt32  VISATimeout)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  This function returns the minimum timeout value to use (in
/// HIFN  milliseconds) when accessing the device associated with the given
/// HIFN  session.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC hmdmm_GetVISATimeout(ViSession instrSession,
                                       ViUInt32* VISATimeout)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/*****************************************************************************
 * Function: Set OPC Timeout
 * Purpose:  Sets new OPC timeout
*****************************************************************************/
ViStatus _VI_FUNC hmdmm_SetOPCTimeout(ViSession instrSession,
							 ViInt32   timeout)
{
	// TODO: ERROR!!! Missing Lock / Unlock
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
	return error;
}

/*****************************************************************************
 * Function: Get OPC Timeout
 * Purpose:  Gets new OPC timeout
*****************************************************************************/
ViStatus _VI_FUNC hmdmm_GetOPCTimeout(ViSession instrSession,
							 ViInt32*  timeout)
{
	// TODO: ERROR!!! Missing Lock / Unlock
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
	return error;
}

/// HIFN This function retrieves and then clears the error information
/// HIFN for the session or the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If there is no description, the function returns an empty string.
ViStatus _VI_FUNC hmdmm_GetError(ViSession instrSession,
                                 ViStatus* errorCode,
                                 ViInt32   bufferSize,
                                 ViChar    description[])
{
	ViStatus error = VI_SUCCESS;

	// Lock Session - do not jump to the end, even with invalid instrSession the function continues further
	(void)RsCore_LockSession(instrSession);

	// Test for nulls and acquire error data
	viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

	checkErr(RsCore_GetErrorCompleteDescription(instrSession, &hmdmm_utility_error_message, errorCode, bufferSize, description));

Error:  
	(void)RsCore_UnlockSession(instrSession); // IGNORE: ERROR!!! Missing Lock
	return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
ViStatus _VI_FUNC hmdmm_ClearError(ViSession instrSession)
{
	return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC hmdmm_WriteInstrData(ViSession instrSession,
                                       ViString  writeBuffer)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	error = RsCore_Write(instrSession, writeBuffer);

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC hmdmm_QueryViBoolean(ViSession  instrSession,
                                       ViString   cmd,
                                       ViBoolean* value)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2, "Command (null string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

	checkErr(RsCore_QueryViBoolean(instrSession, cmd, value));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC hmdmm_QueryViInt32(ViSession instrSession,
                                     ViString  cmd,
                                     ViInt32*  value)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2, "Command (null string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

	checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC hmdmm_QueryViReal64(ViSession instrSession,
                                      ViString  cmd,
                                      ViReal64* value)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2, "Command (null string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

	checkErr(RsCore_QueryViReal64(instrSession, cmd, value));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC hmdmm_QueryViString(ViSession instrSession,
                                      ViString  cmd,
                                      ViInt32   bufferSize,
                                      ViChar    value[])
{
	ViStatus error = VI_SUCCESS;
	ViChar*  pbuffer = NULL;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2, "Command (null string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

	checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

	checkErr(hmdmm_CheckStatus(instrSession));

	checkErr(RsCore_CopyToUserBufferAsciiData(value, bufferSize, pbuffer));

Error:
	if (pbuffer) free(pbuffer);
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN  Sends a command to the instrument synchronised with OPC-polling. Use
/// HIFN  this function to wait for an instrument operation that can take a long
/// HIFN  time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value in ms. Set it to 0 if you don't
/// HIPAR timeout/want the function to wait until the acquisition is complete.
ViStatus _VI_FUNC hmdmm_WriteCommandWithOPCSync(ViSession instrSession,
                                                ViString  writeBuffer,
                                                ViUInt32  timeout)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidStringLength(instrSession, writeBuffer, 1, -1),
		2, "Write Buffer (string length)");

	checkErr(RsCore_WriteWithOpc(instrSession, writeBuffer, timeout));

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN  Sends a query to the instrument synchronised with OPC-polling. Use
/// HIFN  this function to wait for a response that can take a long time to
/// HIFN  complete e.g. self-alignment, calibration or self-test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value in ms. Set it to 0 if you don't
/// HIPAR timeout/want the function to wait until the acquisition is complete.
/// HIPAR bufferSize/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data that
/// HIPAR readBuffer/was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument and
/// HIPAR numBytesRead/stored in the Read Buffer.
ViStatus _VI_FUNC hmdmm_QueryWithOPCSync(ViSession instrSession,
                                         ViString  writeBuffer,
                                         ViUInt32  timeout,
                                         ViInt32   bufferSize,
                                         ViChar    readBuffer[],
                                         ViUInt32* numBytesRead)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	viCheckParm(RsCore_InvalidStringLength(instrSession, writeBuffer, 1, -1),
		2, "Write Buffer (string length)");
	viCheckParm(RsCore_InvalidNullPointer(instrSession, readBuffer),
		5, "Read Buffer");

	checkErr(RsCore_QueryViStringWithOpc(instrSession, writeBuffer, timeout, bufferSize, readBuffer));

	*numBytesRead = (ViUInt32)strlen(readBuffer);

	checkErr(hmdmm_CheckStatus(instrSession));

Error:
	(void)RsCore_UnlockSession(instrSession);
	return error;
}


/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.
ViStatus _VI_FUNC hmdmm_close(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	checkErr(RsCore_LockSession(instrSession));

	checkErr(RsCore_ViClose(instrSession));

Error:
	(void)RsCore_Dispose(instrSession);
	(void)RsCore_UnlockSession(instrSession);
	return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function hmdmm_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC hmdmm_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery)
{
	ViStatus         error = VI_SUCCESS;
	RsCoreSessionPtr rsSession = NULL;

	checkErr(RsCore_GetRsSession(instrSession, &rsSession));
	rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
	return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the hmdmm_init or
/// HIPAR instrSession/hmdmm_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC hmdmm_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking)
{
	ViStatus         error = VI_SUCCESS;
	RsCoreSessionPtr rsSession = NULL;

	checkErr(RsCore_GetRsSession(instrSession, &rsSession));
	rsSession->multiThreadLocking = multiThreadLocking;

Error:
	return error;
}

///HINF  Switches OPC-synchronization ON/OFF for writes.
///HINF  
///HIRET Returns the status code of this operation.
///HIPAR enable/Switches OPC-synchronization ON/OFF for writes.
ViStatus _VI_FUNC hmdmm_OPCSyncWriteEnable(
    ViSession instrSession,
    ViBoolean enable
)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->opcSyncAttrWrites = enable;

Error:
    return error;
}

///HINF  Switches OPC-synchronization ON/OFF for queries.
///HINF  
///HIRET Returns the status code of this operation.
///HIPAR enable/Switches OPC-synchronization ON/OFF for queries.
ViStatus _VI_FUNC hmdmm_OPCSyncQueryEnable(
    ViSession instrSession,
    ViBoolean enable
)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->opcSyncAttrQueries = enable;

Error:
    return error;
}

///HINF  Sets the mechanism which is used for OPC-synchronised queries
///HINF  (valid for read attributes and all OPC-synchronised queries):
///HINF  Default value: 'useEsb' = true, 'clsBefore' = true, 'useMav' =
///HINF  false
///HINF  
///HINF  Scenario A: useEsb = true:
///HINF  The query command is appended by the ';*OPC', and the wait
///HINF  loop ends if the ESB bit signals that the command was completed,
///HINF  or the ERRQ (bit 2) signals an error.
///HINF      - clsBefore has no effect and is always set to true.
///HINF      - set the 'useMav' to true, to enable the opc-wait loop
///HINF  sucessfully finish if the MAV bit signals, that the response is
///HINF  available, even if the ESB bit is still 0.
///HINF  
///HINF  Scenario B: useEsb = false:
///HINF  The query command is not appended by the ';*OPC', and the wait
///HINF  loop ends if the MAV bit signals that the message is
///HINF  available, or the ERRQ (bit 2) signals an error.
///HINF      - clsBefore causes sending query *STB? before sending the
///HINF  command to clear the error queue and output buffers.
///HINF      - useMav has no effect, and is always set to true.
///HINF  
///HIRET Returns the status code of this operation.
///HIPAR useEsb/Switches queries OPC-synchronization to use the Event
///HIPAR useEsb/Status Bit.
///HIPAR clsBefore/Switches queries OPC-synchronization to use call
///HIPAR clsBefore/the Clear Before Read procedure before sending the
///HIPAR clsBefore/query.
///HIPAR useMav/Switches queries OPC-synchronization to react on
///HIPAR useMav/Message Available bit (MAV)
ViStatus _VI_FUNC hmdmm_SetOpcSyncQueryMechanism(
    ViSession instrSession,
    ViBoolean useEsb,
    ViBoolean clsBefore,
    ViBoolean useMav
)
{
    return RsCore_SetOpcSyncQueryMechanism(instrSession, useEsb, clsBefore, useMav);
}

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
