/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include "hmdmm.h"
#include <time.h>

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif

#ifndef ANSI                // UNIX compatible
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       // ANSI compatible
#include <windows.h>
#endif

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- Common Callback Declarations --------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function: hmdmm_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
ViStatus hmdmm_InitAttributes(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	// - Driver Identification -
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, HMDMM_SPECIFIC_DRIVER_DESCRIPTION));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, HMDMM_SPECIFIC_DRIVER_PREFIX));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, HMDMM_SPECIFIC_DRIVER_LOCATOR));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, HMDMM_SPECIFIC_DRIVER_VENDOR));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, HMDMM_CLASS_SPEC_MAJOR_VERSION));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, HMDMM_CLASS_SPEC_MINOR_VERSION));

	// - Driver Capabilities -
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, HMDMM_SUPPORTED_INSTRUMENT_MODELS));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, HMDMM_GROUP_CAPABILITIES));
	checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, HMDMM_FUNCTION_CAPABILITIES));

	// - Version Info -
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, HMDMM_MAJOR_VERSION));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, HMDMM_MINOR_VERSION));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, HMDMM_MINOR_MINOR_VERSION));

	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, HMDMM_CLASS_SPEC_MAJOR_VERSION));
	checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, HMDMM_CLASS_SPEC_MINOR_VERSION));

	checkErr(RsCore_SetSpecificDriverRevision(instrSession, RS_ATTR_SPECIFIC_DRIVER_REVISION));

Error:
	return error;
}

/*****************************************************************************
 * Function: hmdmm_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           hmdmm_reset function calls this function. The
 *           hmdmm_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus hmdmm_DefaultInstrSetup(ViSession instrSession)
{
	ViStatus         error = VI_SUCCESS;
	RsCoreSessionPtr rsSession = NULL;

	checkErr(RsCore_GetRsSession(instrSession, &rsSession));
	// Set all the attributes to the default state. Do not update inherent attributes!
	checkErr(RsCore_ApplyAttributeDefaults(instrSession, VI_FALSE));

	checkErr(hmdmm_InitAttributes(instrSession));
	rsSession->fastSweepInstrument = 0;
	checkErr(RsCore_ResetRegistersEseSre(instrSession));

	if (!RsCore_Simulating(instrSession))
	{
		checkErr(RsCore_ClearStatus(instrSession));
	}

Error:
	return error;
}

/*****************************************************************************
 * Function: hmdmm_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           hmdmm_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus hmdmm_CheckStatus(ViSession instrSession)
{
	ViStatus error = VI_SUCCESS;

	if (RsCore_QueryInstrStatus(instrSession) && !RsCore_Simulating(instrSession))
	{
		checkErr(RsCore_CheckStatus(instrSession, VI_SUCCESS));
	}

Error:
	return error;
}

// Queries the instrument repeatedly until the result is not NAN or the timeout occures
ViStatus hmdmm_MeasureRepeated(ViSession     instrSession,
                               ViConstString query,
                               ViInt32       timeout,
                               ViReal64*     value)
{  
// TODO: ERROR!!! Missing Lock / Unlock
	ViStatus error = VI_SUCCESS;
	ViUInt32 oldVisaTimeout = 0;
	ViInt32  loopTimeout;
	ViReal64 elapsedTimeMs;

	// ReSharper disable once CppEqualOperandsInBinaryExpression
	ViReal64 ms_divider = (ViReal64)CLOCKS_PER_SEC / (ViReal64)1000.0;
	clock_t start_time = clock();

	if (timeout == 0)
	{
		checkErr(hmdmm_GetOPCTimeout(instrSession, &loopTimeout));
	}
	else
	{
		checkErr(hmdmm_GetVISATimeout(instrSession, &oldVisaTimeout));
		if ((ViInt32)oldVisaTimeout != timeout)
		{
			checkErr(hmdmm_SetVISATimeout(instrSession, timeout));
		}
		else
		{
			oldVisaTimeout = 0;
		}

		loopTimeout = timeout;
	}

	do
	{
		checkErr(RsCore_QueryViReal64(instrSession, query, value));

		if (*value < 9E30)
			goto Error;

		elapsedTimeMs = (ViReal64)(clock() - start_time) / ms_divider;
		if (elapsedTimeMs > loopTimeout)
		{
			error = VI_ERROR_TMO;
			goto Error;
		}

		Sleep(5);
	}
	while (1);

Error:
	if (oldVisaTimeout > 0)
	{
		hmdmm_SetVISATimeout(instrSession, oldVisaTimeout);
	}

	return error;
}

ViStatus hmdmm_utility_error_message(ViSession instrSession,
                                     ViStatus  errorCode,
                                     ViChar    errorMessage[512])
{
	ViStatus error = VI_SUCCESS;

	static RsCoreStringValueTable errorTable =
	{
		HMDMM_ERROR_CODES_AND_MSGS,
		{ 0, NULL }
	};

	(void)RsCore_LockSession(instrSession);

	// all VISA and RS error codes are handled as well as codes in the table
	viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

	checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:  
	(void)RsCore_UnlockSession(instrSession); // IGNORE: ERROR!!! Missing Lock
	return error;
}
